sp|D4GVD7|CETZ1_HALVD Tubulin-like protein CetZ1 OS=Haloferax volcanii (strain ATCC 29605 / DSM 3757 / JCM 8879 / NBRC 14742 / NCIMB 2012 / VKM B-1768 / DS2) GN=cetZ1 PE=1 SV=1
sp|D4GTC1|CETZ2_HALVD Tubulin-like protein CetZ2 OS=Haloferax volcanii (strain ATCC 29605 / DSM 3757 / JCM 8879 / NBRC 14742 / NCIMB 2012 / VKM B-1768 / DS2) GN=cetZ2 PE=1 SV=1
sp|A0B5R2|CETZ_METTP Tubulin-like protein CetZ OS=Methanosaeta thermophila (strain DSM 6194 / JCM 14653 / NBRC 101360 / PT) GN=cetZ PE=1 SV=1
sp|D4GU26|CETZ5_HALVD Tubulin-like protein CetZ5 OS=Haloferax volcanii (strain ATCC 29605 / DSM 3757 / JCM 8879 / NBRC 14742 / NCIMB 2012 / VKM B-1768 / DS2) GN=cetZ5 PE=3 SV=1
sp|D4GW48|CETZ3_HALVD Tubulin-like protein CetZ3 OS=Haloferax volcanii (strain ATCC 29605 / DSM 3757 / JCM 8879 / NBRC 14742 / NCIMB 2012 / VKM B-1768 / DS2) GN=cetZ3 PE=3 SV=1
sp|Q9V0H5|CETZ_PYRAB Tubulin-like protein CetZ OS=Pyrococcus abyssi (strain GE5 / Orsay) GN=cetZ PE=3 SV=1
sp|D4GU81|CETZ6_HALVD Tubulin-like protein CetZ6 OS=Haloferax volcanii (strain ATCC 29605 / DSM 3757 / JCM 8879 / NBRC 14742 / NCIMB 2012 / VKM B-1768 / DS2) GN=cetZ6 PE=3 SV=1
sp|D4GQ67|CETZ4_HALVD Tubulin-like protein CetZ4 OS=Haloferax volcanii (strain ATCC 29605 / DSM 3757 / JCM 8879 / NBRC 14742 / NCIMB 2012 / VKM B-1768 / DS2) GN=cetZ4 PE=3 SV=1
sp|O59060|CETZ_PYRHO Tubulin-like protein CetZ OS=Pyrococcus horikoshii (strain ATCC 700860 / DSM 12428 / JCM 9974 / NBRC 100139 / OT-3) GN=cetZ PE=3 SV=1
sp|Q8U0S7|CETZ_PYRFU Tubulin-like protein CetZ OS=Pyrococcus furiosus (strain ATCC 43587 / DSM 3638 / JCM 8422 / Vc1) GN=cetZ PE=3 SV=1
sp|Q9HHD0|CETZ_THEKO Tubulin-like protein CetZ OS=Thermococcus kodakarensis (strain ATCC BAA-918 / JCM 12380 / KOD1) GN=cetZ PE=3 SV=1
